/*
 * This file is part of decoding
 * Copyright 2012  Guillaume Quintin, INRIA
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 */

/* low-level structs */
struct ll_upol_shift_fast_pc { /* pc is for PreComputations */
  R **pow;
  int npow;
};

struct ll_upol_shift_car2_pc {
  R *pow;
  int npow;
};

struct ll_dbpol_shift_fast_pc {
  struct ll_upol_shift_fast_pc *powX,*powY;
};

struct ll_dbpol_shift_car2_pc {
  struct ll_upol_shift_car2_pc *powX,*powY;
};

struct ll_lili_vec {
  R *a;
  int na;
  struct ll_lili_vec *next,*prev;
};

struct ll_upol_prod_tree {
  R *a;
  int na;
  struct ll_upol_prod_tree *left,*right;
};

/* algos functions */

/* algos/array.c */
void ll_array_add(R*,R*,int,R*,int);
void ll_array_add_mul_by_cte(R*,int,R*,int,R);

/* algos/dbpol_horner.c */
void ll_dbpol_horner(R,R*,int,int,R,R);
void ll_dbpol_horner_upol(R*,int*,R*,int,int,R*,int);

/* algos/dbpol_mul_by_affine.c */
void ll_dbpol_mul_by_affine(R*,R*,int,int,R);

/* algos/dbpol_mul_by_upol.c */
void ll_dbpol_mul_by_upolX(R*,R*,int,int,R*,int);
void ll_dbpol_mul_by_upolY(R*,R*,int,int,R*,int);

/* algos/dbpol_mul_by_Y.c */
void ll_dbpol_mul_by_Y(R*,R*,int,int);
void ll_dbpol_mul_by_Yn(R*,R*,int,int,int);

/* algos/dbpol_mul.c */
void ll_dbpol_mul_naive(R*,R*,int,int,R*,int,int);
void ll_dbpol_mul(R*,R*,int,int,R*,int,int);

/* algos/dbpol_print.c */
void ll_dbpol_print_factorized(FILE*,R*,int,int);

/* algos/dbpol_random.c */
R *ll_dbpol_malloc_random(int,int);

/* algos/dbpol_replace_X_by_XY.c */
void ll_dbpol_replace_X_by_XYn(R*,int*,R*,int,int,int);

/* algos/dbpol_shift.c */
void ll_dbpol_shift_naive(R*,R*,int,int,R,R);
void ll_dbpol_shift(R*,R*,int,int,R,R);

/* algos/dbpol_shift_fast.c */
void ll_dbpol_shift_fast_rec(R*,R*,int,int,R,R**,R*);
void ll_dbpol_shift_fast2(R*,R*,int,int,R,R,
                          struct ll_dbpol_shift_fast_pc*);
struct ll_dbpol_shift_fast_pc *ll_dbpol_shift_fast_prepare(int,int,R,R);
void ll_dbpol_shift_fast(R*,R*,int,int,R,R);
void ll_dbpol_shift_fast_finish(struct ll_dbpol_shift_fast_pc*);
void ll_dbpol_shift_car2_rec(R*,int,int,R,R*);
void ll_dbpol_shift_car22(R*,R*,int,int,R,R,
                          struct ll_dbpol_shift_car2_pc*);
struct ll_dbpol_shift_car2_pc *ll_dbpol_shift_car2_prepare(int,int,R,R);
void ll_dbpol_shift_car2(R*,R*,int,int,R,R);
void ll_dbpol_shift_car2_finish(struct ll_dbpol_shift_car2_pc*);

/* algos/dbpol_shift_in_X.c */
void ll_dbpol_shift_in_X(R*,R*,int,int,R);

/* algos/dbpol_Xroots.c */
struct ll_lili_vec *ll_dbpol_Xroots_rec(R*,int,int,int,int,int);
struct ll_lili_vec *ll_dbpol_Xroots(R*,int,int,int);

/* algos/koetter.c */
int ll_koetter_single(R*,int*,int*,R*,int,int,int,int,R*,int);
int ll_koetter_multi(R*,int*,int*,R*,int,int,int,int*,R*,int);

/* algos/linked_list_vec.c */
struct ll_lili_vec *ll_lili_vec_node_malloc(int);
struct ll_lili_vec *ll_lili_vec_node_malloc_copy(R*,int);
void ll_lili_vec_node_free(struct ll_lili_vec*);
void ll_lili_vec_node_attach_after(struct ll_lili_vec*,
                                   struct ll_lili_vec*);
void ll_lili_vec_node_attach_before(struct ll_lili_vec*,
                                    struct ll_lili_vec*);
void ll_lili_vec_node_detach(struct ll_lili_vec*);
void ll_lili_vec_list_free(struct ll_lili_vec*);

/* algos/mat.c */
void ll_mat_copy(R*,R*,int,int);
R *ll_mat_malloc_init(int,int);
R *ll_mat_malloc_random(int,int);
void ll_mat_free(R*,int,int);
void ll_mat_transpose(R*,R*,int,int);

/* algos/memory.c */
void *ll_malloc(int);
void *ll_realloc(void*,int);
void *ll_calloc(int);
void ll_free(void*);

/* algos/print.c */
void ll_print(FILE*,const char*,...);

/* algos/upol_eucldiv.c */
void ll_upol_quorem_naive(R*,int*,R*,R*,int,R*,int);
void ll_upol_quo_naive(R*,R*,int,R*,int);
void ll_upol_rem_naive(R*,int*,R*,int,R*,int);
void ll_upol_quorem(R*,int*,R*,R*,int,R*,int);
void ll_upol_quo(R*,R*,int,R*,int);
void ll_upol_rem(R*,int*,R*,int,R*,int);

/* algos/upol_gcd.c */
void ll_upol_gcd_naive(R*,int*,R*,int,R*,int);
void ll_upol_gcd(R*,int*,R*,int,R*,int);

/* algos/upol_horner.c */
void ll_upol_horner(R,R*,int,R);
void ll_upol_multi_horner(R*,R*,int,R*,int);

/* algos/upol_mul_by_affine.c */
void ll_upol_mul_by_affine(R*,R*,int,R);

/* algos/upol_mul_by_X.c */
void ll_upol_mul_by_X(R*,R*,int);
void ll_upol_mul_by_Xn(R*,R*,int,int);

/* algos/upol_mul.c */
void ll_upol_mul_naive(R*,R*,int,R*,int);
void ll_upol_mul(R*,R*,int,R*,int);

/* algos/upol_normalize.c */
void ll_upol_normalize(R*,int);

/* algos/upol_print.c */
void ll_upol_print(FILE*,R*,int);

/* algos/upol_random.c */
void ll_upol_random(R*,int);
R *ll_upol_malloc_random(int);

/* algos/upol_roots.c */
void ll_upol_roots(R*,int*,R*,int);

/* algos/upol_shift.c */
void ll_upol_shift_naive(R*,R*,int,R);
void ll_upol_shift(R*,R*,int,R);

/* algos/upol_shift_fast.c */
void ll_upol_shift_fast_rec(R*,R*,int,R,R**,R*);
void ll_upol_shift_fast2(R*,R*,int,R,struct ll_upol_shift_fast_pc*);
struct ll_upol_shift_fast_pc *ll_upol_shift_fast_prepare(int,R);
void ll_upol_shift_fast_finish(struct ll_upol_shift_fast_pc*);
void ll_upol_shift_fast(R*,R*,int,R);
void ll_upol_shift_car2_rec(R*,R*,int,R,R*);
void ll_upol_shift_car22(R*,R*,int,R,struct ll_upol_shift_car2_pc*);
struct ll_upol_shift_car2_pc *ll_upol_shift_car2_prepare(int,R);
void ll_upol_shift_car2_finish(struct ll_upol_shift_car2_pc*);
void ll_upol_shift_car2(R*,R*,int,R);

/* algos/upol_square.c */
void ll_upol_square_naive(R*,R*,int);
void ll_upol_square(R*,R*,int);

/* algos/utils.c */
void ll_die(const char*);
int ll_min(int,int);
int ll_max(int,int);
int ll_ceil_div(int,int);
int ll_log2(int);
int ll_prev_log2(int);

/* algos/vec.c */
void ll_vec_add(R*,R*,R*,int);
void ll_vec_sub(R*,R*,R*,int);
void ll_vec_sub_sub(R*,R*,R*,int);
void ll_vec_add_mul(R*,R*,R*,int);
void ll_vec_mul_by_cte(R*,R*,int,R);
void ll_vec_add_mul_by_cte(R*,R*,int,R);
void ll_vec_sub_mul(R*,R*,R*,int);
void ll_vec_init(R*,int);
void ll_vec_init_random(R*,int);
void ll_vec_clear(R*,int);
int ll_vec_hamming_weight(R*,int);
void ll_vec_zero(R*,int);
int ll_vec_iszero(R*,int);
void ll_vec_random(R*,int);
void ll_vec_random_hamming_weight(R*,int,int);
void ll_vec_copy(R*,R*,int);
R *ll_vec_malloc_init(int);
R *ll_vec_realloc(R*,int,int);
R *ll_vec_malloc_copy(R*,int);
R *ll_vec_malloc_random(int);
void ll_vec_free(R*,int);
int ll_vec_cmp(R*,R*,int);
void ll_vec_print(FILE*,R*,int);

/* algos/upol_prod_affine.c */
void ll_upol_prod_affine(R*,R*,int);
void ll_upol_prod_affine_neg(R*,R*,int);

/* algos/dbpol_upol_map_XtoY.c */
void ll_dbpol_upol_map_XtoY(R*,int,R*,int);

