/*
 * This file is part of decoding
 * Copyright 2012  Guillaume Quintin, INRIA
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 */

/**
> `void ll_array_add(R *r,R *a,int na,R *b,int nb);`
  Set `r` to `a + b` where `na` and `nb` are the sizes of `a` and `b`.
  If `a` and `b` have not the same size the shortest is considered to
  be extended with zeros.
  You can have `r == a` and/or `r == b`.
*/

void ll_array_add(R *r,R *a,int na,R *b,int nb) {
  if ( na < nb ) {
    ll_vec_add(r,a,b,na);
    ll_vec_copy(&r[na],&b[na],nb - na);
  }
  else {
    ll_vec_add(r,a,b,nb);
    ll_vec_copy(&r[nb],&a[nb],na - nb);
  }
}

/**
> `void ll_array_add_mul_by_cte(R *r,int nr,R *a,int na,R b);`
  Set `r` to `r + (a * b)` where `nr` and `na` are the sizes of
  `r` and `a`.
  If `r` and `a` have not the same size the shortest is considered
  extended with zeros.
  You can have `r == a`, `b` must not be in `r` or `a`.
*/

void ll_array_add_mul_by_cte(R *r,int nr,R *a,int na,R b) {
  if ( nr == na ) {
    ll_vec_add_mul_by_cte(r,a,nr,b);
  }
  else if ( nr < na ) {
    ll_vec_add_mul_by_cte(r,a,nr,b);
    ll_vec_mul_by_cte(&r[nr],&a[nr],na - nr,b);
  }
  else {
    ll_vec_add_mul_by_cte(r,a,na,b);
  }
}

