/*
 * This file is part of decoding
 * Copyright 2012  Guillaume Quintin, INRIA
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 */

/**
> `void ll_dbpol_horner(R z,R *a,int nax,int nay,R x,R y);`
  Evaluate `a` of size (`nax`,`nay`).
  The Horner algorihtm is used.
*/

void ll_dbpol_horner(R z,R *a,int nax,int nay,R x,R y) {
  int i;
  R *tmp;
  tmp = ll_vec_malloc_init(nay);
  for( i = 0 ; i < nay ; i++ ) {
    ll_upol_horner(tmp[i],&a[i * nax],nax,x);
  }
  ll_upol_horner(z,tmp,nay,y);
  ll_vec_free(tmp,nay);
}

/**
> `void ll_dbpol_horner_upol(R *r,int *nr,R *a,int nax,int nay\
  ,R *b,int nb);`
  Evaluate `a` of size (`nax`,`nay`).
  The Horner algorithm is used.
*/

void ll_dbpol_horner_upol(R *r,int *nr,
                          R *a,int nax,int nay,R *b,int nb)
{
  R *s,*t,*t0,*xchg;
  int i,j,ns,nt0;
  s = r;
  nt0 = (nay - 1) * (nb - 1) + (nax - 1) + 1;
  t0 = ll_vec_malloc_init(nt0);
  t = t0;
  i = (nay - 1) * nax;
  ll_vec_copy(s,&a[i],nax);
  ns = nax;
  for( i -= nax ; i >= 0 ; i -=nax ) {
    ll_upol_mul(t,s,ns,b,nb);
    ns += nb - 1;
    for( j = ns - 1 ; j >= 0 && R_iszero(t[j]) ; j-- );
    ns = j + 1;
    ll_array_add(t,t,ns,&a[i],nax);
    ns = ll_max(ns,nax);
    xchg = s;
    s = t;
    t = xchg;
  }
  if ( s != r ) ll_vec_copy(r,s,ns);
  ll_vec_free(t0,nt0);
  nr[0] = ns;
}

