/*
 * This file is part of decoding
 * Copyright 2012  Guillaume Quintin, INRIA
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 */

/**
> `void ll_dbpol_shift_naive(r *r,r *a,int nax,int nay,r x,r y);`
  Set `r` to `a(X + x,Y + y)` where `a` has size (`nax`,`nay`).
  The horner algorithm is used.
*/

void ll_dbpol_shift_naive(R *r,R *a,int nax,int nay,R x,R y) {
  int i,j;
  R *tmp;
  tmp = ll_mat_malloc_init(nax,nay);
  for( i = 0 ; i < nay ; i++ ) {
    ll_upol_shift(&tmp[i * nax],&a[i * nax],nax,x);
  }
  ll_vec_copy(r,&tmp[(nay - 1) * nax],nax);
  for( i = nay - 2,j = 1 ; i >= 0 ; i--,j++ ) {
    ll_dbpol_mul_by_affine(r,r,nax,j,y);
    ll_vec_add(r,r,&tmp[i * nax],nax);
  }
  ll_mat_free(tmp,nax,nay);
}

/**
> `void ll_dbpol_shift_naive(r *r,r *a,int nax,int nay,r x,r y);`
  Set `r` to `a(X + x,Y + y)` where `a` has size (`nax`,`nay`).
  You can provide your own function with `MY_DBPOL_SHIFT`.
  If your ring has caracteristic 2 (`RING_COMM_CAR2`) then
  `ll_dbpol_shift_car2` is called and `ll_dbpol_shift_fast`
  otherwise.
*/

void ll_dbpol_shift(R *r,R *a,int nax,int nay,R x,R y) {
#ifdef MY_DBPOL_SHIFT
  MY_DBPOL_SHIFT(r,a,nax,nay,x,y);
#else
#ifdef RING_COMM_CAR2
  ll_dbpol_shift_car2(r,a,nax,nay,x,y);
#else
  ll_dbpol_shift_fast(r,a,nax,nay,x,y);
#endif
#endif
}

