/*
 * This file is part of decoding
 * Copyright 2012  Guillaume Quintin, INRIA
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 */

/**
> `void ll_mat_copy(R *r,R *a,int ra,int ca);`
  Do a deep copy of `a` of size (`ar`,`ac`) to `r`.
*/

void ll_mat_copy(R *r,R *a,int ra,int ca) {
  ll_vec_copy(r,a,ra * ca);
}

/**
> `R *ll_mat_malloc_init(int rm,int cm);`
  Return a pointer to a region of (`rm x cm`) `R`'s.
*/

R *ll_mat_malloc_init(int rm,int cm) {
  return ll_vec_malloc_init(rm * cm);
}

/**
> `R *ll_mat_malloc_random(int mr,int mc);`
  Return a pointer to a region of (`rm x cm`) random `R`'s.
*/

R *ll_mat_malloc_random(int rm,int cm) {
  return ll_vec_malloc_random(rm * cm);
}

/**
> `void ll_mat_free(R *m,int rm,int cm);`
  Free `m` of size `rm x cm`.
*/

void ll_mat_free(R *m,int rm,int cm) {
  ll_vec_free(m,rm * cm);
}

/**
> `void ll_mat_transpose(R *r,R *a,int ra,int ca);`
  Set `r` to be the transpose of `a` which has size (`ra`,`ca`).
  You can have `r == a` for `ra == ca`.
*/

void ll_mat_transpose(R *r,R *a,int ra,int ca) {
  int i,j,k,na;
  R *tmp;
  na = ra * ca;
  if ( a == r ) {
    tmp = ll_vec_malloc_copy(a,na);
  }
  else {
    tmp = a;
  }
  for( i = 0,j = 0,k = 0 ; i < na ; i++,j += ra ) {
    if ( j >= na ) {
      k++;
      j = k;
    }
    R_copy(r[j],tmp[i]);
  }
}

