/*
 * This file is part of decoding
 * Copyright 2012  Guillaume Quintin, INRIA
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 */

/**
> `void *ll_malloc(int n);`
  Allocate `n` bytes of memory and return a pointer to the allocated
  region.
  If any error occur during `malloc` the `SIGABRT` signal is raised
  by the `abort` function.
*/

void *ll_malloc(int n) {
  void *p;
  p = malloc(n);
  if ( !p ) {
    fprintf(stderr,"decoding: cannot malloc %d bytes\n",n);
    abort();
  }
  return p;
}

/**
> `void *ll_realloc(void *ptr,int n);`
  Change the size of the memory block pointed to by `ptr` to `n` bytes.
  If any error occur during `malloc` the `SIGABRT` signal is raised
  by the `abort` function.
*/

void *ll_realloc(void *ptr,int n) {
  void *p;
  p = realloc(ptr,n);
  if ( !p ) {
    fprintf(stderr,"decoding: cannot realloc %d bytes\n",n);
    abort();
  }
  return p;
}

/**
> `void *ll_calloc(int n);`
  Allocate `n` bytes of memory and return a pointer to the allocated
  region. The memory is set to zero.
  If any error occur during `malloc` the `SIGABRT` signal is raised
  by the `abort` function.
*/

void *ll_calloc(int n) {
  void *p;
  p = calloc(1,n);
  if ( !p ) {
    fprintf(stderr,"decoding: cannot calloc %d bytes\n",n);
    abort();
  }
  return p;
}

/**
> `void ll_free(void *p);`
  Free the memory space pointed to by `p`.
*/

void ll_free(void *p) {
  free(p);
}

