/*
 * This file is part of decoding
 * Copyright 2012  Guillaume Quintin, INRIA
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 */

/**
> `void ll_upol_square_naive(R *r,R *a,int na);`
  Set `r` to `a(X)^2` where `a` has size `na`.
  The schoolbook algorithm is used.
*/

void ll_upol_square_naive(R *r,R *a,int na) {
  int i,j,k,nr;

  nr = 2 * na - 1;
  for( i = 0 ; i < nr ; i++ ) {
    j = i - (na - 1);
    if ( j < 0 ) j = 0;
    k = i - j;
    R_zero(r[i]);
    for( ; k >= 0 && j < na ; j++,k-- ) {
      R_add_mul(r[i],a[j],a[k]);
    }
  }
}

/**
> `void ll_upol_square(R *r,R *a,int na);`
  Set `r` to `a(X)^2` where `a` has size `na`.
  The schoolbook algorithm is used, `ll_upol_square_naive` is
  called.
*/

void ll_upol_square(R *r,R *a,int na) {
  ll_upol_square_naive(r,a,na);
}

