/*
 * This file is part of decoding
 * Copyright 2012  Guillaume Quintin, INRIA
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 */

/**
> `void ll_die(const char *msg);`
  Print `msg` to `stderr` and terminate the program
  with exit code `-1`.
*/

void ll_die(const char *msg) {
#ifndef R
  fprintf(stderr,"%s\n",msg);
#else
#define __S1(a) #a
#define __S(a)  __S1(a)
  fprintf(stderr,__S(R) ": %s\n",msg);
#undef __S
#undef __S1
#endif
  exit(-1);
}

/**
> `int ll_min(int a,int b);`
  Return the lesser value of `a` and `b`.
*/

int ll_min(int a,int b) {
  if ( a < b ) return a; else return b;
}

/**
> `int ll_max(int a,int b);`
  Return the greater value of `a` and `b`.
*/

int ll_max(int a,int b) {
  if ( a > b ) return a; else return b;
}

/**
> `int ll_ceil_div(int a,int b);`
  Return the smallest integral value not less than `a / b`.
*/

int ll_ceil_div(int a,int b) {
  int r;
  r = a / b;
  if ( (a % b) > 0 ) return r + 1;
  return r;
}

#define LT \
    0,0,1,1,2,2,2,2,3,3,3,3,3,3,3,3, \
    4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4, \
    5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5, \
    5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5, \
    6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6, \
    6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6, \
    6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6, \
    6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6, \
    7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7, \
    7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7, \
    7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7, \
    7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7, \
    7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7, \
    7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7, \
    7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7, \
    7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7

/**
> `int ll_log2(int a);`
  Return the base-2 logarithm of `a`.
  If `a == 0`, return `-1`.
*/

int ll_log2(int a) {
  int b,i;
  int lt[] = { LT };
  for( i = (sizeof(int) - 1) * 8 ; i >= 0 ; i -= 8 ) {
    b = (a >> i);
    if ( b ) return lt[b] + i;
  }
  return -1;
}

/**
> `int ll_prev_log2(int a);`
  Return the largest integral value not greater than
  the base-2 logarithm of `a`.
  If `a == 0`, return `-1`.
*/

int ll_prev_log2(int a) {
  int b,i,log2;
  int lt[] = { LT };
  for( i = (sizeof(int) - 1) * 8 ; i >= 0 ; i -= 8 ) {
    b = (a >> i);
    if ( b ) {
      log2 = lt[b] + i;
      if ( a == (1 << log2) ) return log2 - 1; else return log2;
    }
  }
  return -1;
}

#undef LT

