/*
 * This file is part of decoding
 * Copyright 2012  Guillaume Quintin, INRIA
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 */

/**
==OVERVIEW==

It is easy to control what algorithms are used by `decoding`, for
example if you want to replace unvariate polynomial multiplication
with your own function or if the ring you are implementing has
caracteristic 2.

==DESCRIPTION==

The file `include/decoding/rings/GF17.c` is an example of how to
use macros defined below.

> `#define R NAME`
  `R` contains the name `NAME` of the ring you are implementing. See
  `decoding-rings`(3) for details.
*/

#ifdef R
#undef R
#endif

/**
> `#define RING_COMM_CAR2`
  Tell `decoding` that your ring has caracteristic 2. It is not
  mandatory to specify it. When defined it activates certain fast
  algorithms specific to caracteristic 2.
*/

#ifdef RING_COMM_CAR2
#undef RING_COMM_CAR2
#endif

/**
> `#define MY_UPOL_MUL myfunc`
  Replace default univariate polynomial multiplication by `myfunc`.
  See `decoding-lowlevels`(3) for the prototype of `my_func`.
*/

#ifdef MY_UPOL_MUL
#undef MY_UPOL_MUL
#endif

/**
> `#define MY_UPOL_SHIFT myfunc`
  Replace default univariate polynomial shifting by `myfunc`.
  See `decoding-lowlevels`(3) for the prototype of `my_func`.
*/

#ifdef MY_UPOL_SHIFT
#undef MY_UPOL_SHIFT
#endif

/**
> `#define MY_DBPOL_MUL myfunc`
  Replace default dense bivariate polynomial multiplication by `myfunc`.
  See `decoding-lowlevels`(3) for the prototype of `my_func`.
*/

#ifdef MY_DBPOL_MUL
#undef MY_DBPOL_MUL
#endif

/**
> `#define MY_DBPOL_SHIFT myfunc`
  Replace default dense bivariate polynomial shfting by `myfunc`.
  See `decoding-lowlevels`(3) for the prototype of `my_func`.
*/

#ifdef MY_DBPOL_SHIFT
#undef MY_DBPOL_SHIFT
#endif

/**
> `#define MY_UPOL_ROOTS myfunc`
  Replace default univariate root finding by `myfunc`.
  See `decoding-lowlevels`(3) for the prototype of `my_func`.
  Note that `decoding` provides no algorithm for univariate root
  finding.
*/

#ifdef MY_UPOL_ROOTS
#undef MY_UPOL_ROOTS
#endif

/**
> `#define GF_MAGMA_DECL "k := GF(p^n)"`
  This macro is useful only for test purpose with MAGMA.
  See `decoding-rings`(3) for details.
*/

#ifdef GF_MAGMA_DECL
#undef GF_MAGMA_DECL
#endif

/**
==AUTHOR==

Written by Guillaume Quintin (coincoin169g@gmail.com).

==SEE ALSO==

`decoding-rings`(3),`decoding-lowlevels`(3),
MAGMA(http://magma.maths.usyd.edu.au/magma/)
*/

