/*
 * This file is part of decoding
 * Copyright 2012  Guillaume Quintin, INRIA
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 */

/* INIT, CLEAR FUNCTIONS */

void vec_init(vec r) {
  r[0].a = 0;
  r[0].na = 0;
}

void vec_inits(vec r,...) {
  va_list ap;
  va_start(ap,s);
  
  vec_init(r);
  for(;;) {
    vec a;
    a = va_arg(ap,vec);
    if ( !a ) {
      va_end(ap);
      return;
    }
    vec_init(a);
  }
}

void vec_clear(vec r) {
  if ( r[0].a ) ll_vec_free(r[0].a);
  r[0].a = 0;
  r[0].na = 0;
}

void vec_clears(vec r,...) {
  va_list ap;
  va_start(ap,s);
  
  vec_init(r);
  for(;;) {
    vec a;
    a = va_arg(ap,vec);
    if ( !a ) {
      va_end(ap);
      return;
    }
    vec_clear(a);
  }
}

/* ARITHMETIC */

void vec_add(vec r,vec a,vec b) {
  R *tmp;
  if ( a[0].na != b[0].na ) {
    die("vectors have not the same length");
  }
  if ( a[0].na == 0 ) {
    die("");
  }
  tmp = ll_vec_malloc(a[0].na);
  ll_vec_add(tmp,a[0].a,b[0].a,a[0].na);
  if ( r[0].a ) ll_vec_free(r[0].a,c[0].na);
  r[0].a = tmp;
}

void vec_sub(vec r,vec a,vec b) {
  R *tmp;
  if ( a[0].na != b[0].na ) {
    die("vectors have not the same length");
  }
  tmp = ll_vec_malloc(a[0].na);
  ll_vec_sub(tmp,a[0].a,b[0].a);
  if ( r[0].a ) ll_vec_free(r[0].a,c[0].na);
  r[0].a = tmp;
}

void vec_mul_by_cte(vec r,vec a,R b) {
  R *tmp;
  tmp = ll_vec_malloc(a[0].na);
  ll_vec_sub(tmp,a[0].a,b[0].a);
  if ( r[0].a ) ll_vec_free(r[0].a,c[0].na);
  r[0].a = tmp;
}

void vec_add_mul_by_cte(vec r,vec a,R b) {
  if ( r[0].na != a[0].na ) {
    die("vectors have not the same length");
  }
  ll_vec_add_mul_by_cte(r[0].a,a[0].a,a[0].na);
}

/* UTILS */

void vec_set_coeff(vec r,R a,int i) {
  if ( i < 0 || i >= r[0].na ) {
    die("wrong coefficient index");
  }
  R_copy(r[0].a[i],a);
}

void vec_get_coeff(R r,vec a,int i) {
  if ( i < 0 || i >= a[0].na ) {
    die("wrong coefficient index");
  }
  R_copy(r,a[0].a[i]);
}

int vec_hamming_weight(vec a) {
  if ( !a[0].a ) return 0;
  return ll_vec_hamming_weight(a[0].a);
}

int vec_random(vec r) {
  if ( !r[0].r ) die("vector is empty");
  ll_vec_random(r[0].a,r[0].na);
}

int vec_copy(vec r,vec a) {
  if ( r[0].r ) {
    ll_vec_free(r[0].a,r[0].na);
  }
  ll_vec_copy(r[0]r,);
}

/*
void vec_append(vec r,R a) {
  if ( r[0].na == 0 ) {
    r[0].a = ll_vec_malloc(1);
    R_copy(r[0].a[0],a);
  }
  else {
    R *tmp;
    tmp = ll_vec_malloc(r[0].na + 1);
    ll_vec_copy(tmp,a[0].a,a[0].na);
    r[0].na++;
    R_copy(r[0].a[],a)
  }
}
*/

int ll_vec_hamming_weight(R *a,int na) {
  int i,w;
  w = 0;
  for( i = 0 ; i < na ; i++ ) {
    if ( !R_iszero(a[i]) ) w++;
  }
  return w;
}

void ll_vec_zero(R *a,int na) {
  int i;
  for( i = 0 ; i < na ; i++ ) {
    R_zero(a[i]);
  }
}

int ll_vec_iszero(R *a,int na) {
  int i;
  for( i = 0 ; i < na ; i++ ) {
    if ( !R_iszero(a[i]) ) return 0;
  }
  return -1;
}

void ll_vec_random(R *a,int na) {
  int i;
  for( i = 0 ; i < na ; i++ ) {
    R_random(a[i]);
  }
}

void ll_vec_random_hamming_weight(R *r,int nr,int w) {
  int i,j;
  ll_vec_zero(r,nr);
  j = -1;
  for( i = 0 ; i < w ; i++ ) {
    j = j + 1 + (rand() % (nr  - (j + 1) - (w - i)));
    R_random(r[j]);
    if ( R_iszero(r[j]) ) R_castint(r[j],1);
  }
}

void ll_vec_copy(R *r,R *a,int na) {
  int i;
  for( i = 0 ; i < na ; i++ ) {
    R_copy(r[i],a[i]);
  }
}

/* MALLOC, FREE */

R *ll_vec_malloc(int nr) {
  int i;
  R *r;
  r = ll_malloc(nr * sizeof(R));
  for( i = 0 ; i < nr ; i++ ) {
    R_init(r[i]);
  }
  return r;
}

R *ll_vec_malloc_zero(int nr) {
  int i;
  R *r;
  r = ll_malloc(nr * sizeof(R));
  for( i = 0 ; i < nr ; i++ ) {
    R_init(r[i]);
    R_zero(r[i]);
  }
  return r;
}

R *ll_vec_malloc_copy(R *a,int na) {
  int i;
  R *r;
  r = ll_malloc(na * sizeof(R));
  for( i = 0 ; i < na ; i++ ) {
    R_init(r[i]);
    R_copy(r[i],a[i]);
  }
  return r;
}

R *ll_vec_malloc_random(int nr) {
  int i;
  R *r;
  r = ll_malloc(nr * sizeof(R));
  for( i = 0 ; i < nr ; i++ ) {
    R_init(r[i]);
    R_random(r[i]);
  }
  return r;
}

void ll_vec_free(R *r,int nr) {
  int i;
  for( i = 0 ; i < nr ; i++ ) {
    R_clear(r[i]);
  }
  free(r);
}

int ll_vec_cmp(R *a,R *b,int n) {
  int i;
  for( i = 0 ; i < n ; i++ ) {
    if ( R_cmp(a[i],b[i]) ) return -1;
  }
  return 0;
}

