/*
 * This file is part of decoding
 * Copyright 2012  Guillaume Quintin, INRIA
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 */



/* standard includes */
#include <stdlib.h>


/* This is mandatory to use decoding */
#include <decoding/decoding.h> 


/* Now you can choose a base ring to work with */
#include <decoding/rings/GF5.c>
/* and then generate all the algorithms provided by decoding
   for the chosen base ring. You can see the result of this
   generation with:
   
   $ cpp -I../include sample1.c | less
*/
#include <decoding/algos.c>


int main(void) {

  /* Each base ring in ``decoding/" provides a type to
     manipulate its elements. Usually the type name is the
     same as the file name providing the base ring. In this
     exemple it is ``GF5". */
  GF5 a,b,c;

  
  /* You must before doing anything with a, b or c,
     initialize a, b and c. All the functions that manipulate
     elements of GF5 will be prefixed by GF5. */
  GF5_init(a);
  GF5_init(b);
  GF5_init(c);


  /* Take random elements of GF(5) and put them in a and b */
  GF5_random(a);
  GF5_random(b);

  
  /* Compute the sum of a and b and put the result in c */
  GF5_add(c,a,b);


  /* Prettyprint the sum of a and b to the standard output
     The format of GF5_ll_print() is the following:
       %e - an element of the base ring (in this case GF(5))
       %p - a univariate polynomial over GF(5)
       %q - a bivariate polynomial over GF(5)
       %v - a vector over GF(5)
     Note that passing an element of GF(5) to GF5_ll_print() is done
     by passing a pointer to this element. This is because
     GF5_ll_print() is a low-level function. Every function prefixed
     by ``GF5_ll_" is a low-level function. They can be used but are
     not desinged to be user-firnedly or to provide a coherent calling
     interface.
     */
  GF5_ll_print(stdout,"%e + %e = %e\n",&a,&b,&c);


  /* When finished with a, b and c you have to free any memory
     associated to them. This is done with the GF5_clear()
     function. */
  GF5_clear(c);
  GF5_clear(b);
  GF5_clear(a);

  return 0;
}

