/*
 * This file is part of decoding
 * Copyright 2012  Guillaume Quintin, INRIA
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 */



#include <stdlib.h>
#include <decoding/decoding.h> 

/* Use GF(5) */
#include <decoding/rings/GF5.c>
#include <decoding/algos.c>


int main(void) {

  /* A vector over GF5 or a univariate polynomial is
     represented in memory as an array of GF5. */
  GF5 f[5],g[5],h[5];


  /* f and g are polynomials of degree 4 or vectors of length 5
     depending on how you use or prettyprint them. We first must,
     as usual, initialize the coefficients of f and g. */
  GF5_ll_vec_init(f,5);
  GF5_ll_vec_init(g,5);
  GF5_ll_vec_init(h,5);


  /* We take random elements to fill f and g. */
  GF5_ll_vec_random(f,5);
  GF5_ll_vec_random(g,5);


  /* We compute the sum of f and g. All low-level function require
     the memory for the result to be allocated before calling
     the function. The memory of the arguments (including the result)
     must not overlap execept in a few cases. */
  GF5_ll_vec_add(h,f,g,5);


  /* We prettyprint f, g and h as vectors */
  GF5_ll_print(stdout,"As vectors\n%v + %v = %v\n",f,5,g,5,h,5);


  /* and as polynomials. */
  GF5_ll_print(stdout,"As vectors\n( %p ) + ( %p ) = %p\n",
                      f,5,g,5,h,5);


  /* Again, when we are finished we free memory. */
  GF5_ll_vec_clear(h,5);
  GF5_ll_vec_clear(g,5);
  GF5_ll_vec_clear(f,5);

  /* NOTE: As the elements of GF(5) can be represented with a single
           machine word, the type GF5 is in fact int[1]. Thus there
           is no need to init and/or clear f and g or any element of
           type GF5. But, in order to write code which will work with
           rings whose binary representations do not fit in a single
           machine word, it is advised to always use these functions.
           The compiler will do appropriate optimizations if there is
           no need to init or clear elements of type GF5. */

  return 0;

}

