/*
 * This file is part of decoding
 * Copyright 2012  Guillaume Quintin, INRIA
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 */



#include <stdlib.h>
#include <decoding/decoding.h> 


/* This exemple shows how to use several
   rings at the same time. */


/* Generate code for GF(5) */
#include <decoding/rings/GF5.c>
#include <decoding/algos.c>


/* Generate code for GF(64) */
#include <decoding/rings/GF64.c>
#include <decoding/algos.c>


/* Of course, it would be possible to impose
   that the decoding/algos.c file to be included
   directly in all rings files (here in
   decoding/rings/GF5.c and decoding/rings/GF64.c)
   to avoid writing ``#include <decoding/algos.c>"
   each time we want a new ring. But, in the future
   it is possible that, algorithms will be separated
   in several modules and a user would only want
   one or two modules. Hence it would be possible to
   type something like:
   
   #include <decoding/rings/GF5.c>
   #include <decoding/algos_for_unique_decoding.c>

   #include <decoding/rings/GF64.c>
   #include <decoding/algos_for_list_decoding.c>

   This would compile only what is necessary for the user
   tasks.

   */


int main(void) {

  /* A vector over GF(5) and over GF(64) */
  GF5 f[5];
  GF64 g[5];


  GF5_ll_vec_init_random(f,5);
  GF64_ll_vec_init_random(g,5);


  GF5_ll_print(stdout, "(over GF5)  f = %v\n",f,5);
  GF64_ll_print(stdout,"(over GF64) g = %v\n",g,5);


  GF5_ll_vec_clear(f,5);
  GF64_ll_vec_clear(g,5);


  return 0;

}

/* To see how to use low-level functions in details
   please, see in the tests/ directory all the .c files.
   In particular, the tests/gs.c file contains an exemple
   of the Guruswami-Sudan algorithm. */

