/*
 * This file is part of decoding
 * Copyright 2012  Guillaume Quintin, INRIA
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 */

#define _POSIX_C_SOURCE 1

#include <time.h>
#include <stdio.h>
#include <stdlib.h>
#include <stdarg.h>

/* #include <decoding/rings/GF17.c> */
#include <decoding/rings/GF64.c>
#include <decoding/algos.c>

int list_size(int m,int n,int tau,int k) {
  int mu,ret;
  mu = n - tau;
  ret = m * mu / (k - 1);
  if ( ((m * mu) % (k - 1)) == 0 ) ret--;
  return ret;
}

void mtest_gs_GF17(int n2) {
  int i,tau,nqx,nqy,L,m,k,n;
  R supp[17],e[17],c[17],y[17],f[2],*q;
  struct ll_lili_vec *roots;
  
  m = 1;
  n = 17;
  k = 2;
  tau = 10;
  L = list_size(m,n,tau,k);
  ll_vec_init(supp,n);
  ll_vec_init(e,n);
  ll_vec_init(c,n);
  ll_vec_init(y,n);
  ll_vec_init(f,k);
  for( i = 0 ; i < n ; i++ ) {
    supp[i][0] = i;
  }
  q = ll_mat_malloc_init(L + 2,m * n + 1);

  for( i = 0 ; i < n2 ; i++ ) {
    ll_vec_random(f,k);
    ll_upol_multi_horner(c,f,k,supp,n);
    ll_vec_random_hamming_weight(e,n,tau);
    ll_vec_add(y,c,e,n);
    if ( ll_koetter_single(q,&nqx,&nqy,supp,n,k,L,m,y,0) ) {
      ll_print(stderr,"ERROR IN KOETTER\n");
      goto mtest_gs_free_stuff;
    }
    roots = ll_dbpol_Xroots(q,nqx,nqy,k);
    if ( !roots ) {
      ll_print(stderr,"ERROR IN ROOT-FINDING\n");
      goto mtest_gs_free_stuff;
    }
    else {
      struct ll_lili_vec *node;
      for( node = roots ; node ; node = node->next ) {
        if ( !ll_vec_cmp(node->a,f,k) ) goto mtest_gs_ok;
      }
      ll_print(stderr,"ERROR !!!\n");
      exit(-1);
mtest_gs_ok:
      ll_lili_vec_list_free(roots);
    }
  }
  ll_print(stderr,"ALL OK !!!\n");
mtest_gs_free_stuff:
  ll_vec_clear(supp,n);
  ll_vec_clear(c,n);
  ll_vec_clear(e,n);
  ll_vec_clear(y,n);
  ll_vec_clear(f,k);
  ll_mat_free(q,L + 2,m * n + 1);
}

void mtest_gs_GF64(int n2) {
  int i,t,tau,nqx,nqy,L,m,k,n;
  R supp[20],e[20],c[20],y[20],f[10],*q;
  struct ll_lili_vec *roots;
  
  n = 20;
  k = 10;
  tau = 5;
  m = 1;
  t = (n - k) / 2;
  L = list_size(m,n,tau,k);
  ll_print(stderr,"n=%d  k=%d  t=%d\n"
                  "tau=%d  m=%d  L=%d\n",n,k,t,tau,m,L);
  ll_vec_init(supp,n);
  ll_vec_init(e,n);
  ll_vec_init(c,n);
  ll_vec_init(y,n);
  ll_vec_init(f,k);
  for( i = 0 ; i < n ; i++ ) {
    supp[i][0] = i;
  }
  q = ll_mat_malloc_init(L + 2,m * n + 1);

  for( i = 0 ; i < n2 ; i++ ) {
    ll_vec_random(f,k);
    ll_upol_multi_horner(c,f,k,supp,n);
    ll_vec_random_hamming_weight(e,n,tau);
    ll_vec_add(y,c,e,n);
    if ( ll_koetter_single(q,&nqx,&nqy,supp,n,k,L,m,y,0) ) {
      ll_print(stderr,"ERROR IN KOETTER\n");
      goto mtest_gs_free_stuff;
    }
    roots = ll_dbpol_Xroots(q,nqx,nqy,k);
    if ( !roots ) {
      ll_print(stderr,"ERROR IN ROOT-FINDING\n");
      goto mtest_gs_free_stuff;
    }
    else {
      struct ll_lili_vec *node;
      for( node = roots ; node ; node = node->next ) {
        if ( !ll_vec_cmp(node->a,f,k) ) goto mtest_gs_ok;
      }
      ll_print(stderr,"ERROR !!!\n");
      exit(-1);
mtest_gs_ok:
      ll_lili_vec_list_free(roots);
    }
  }
  ll_print(stderr,"ALL OK !!!\n");
mtest_gs_free_stuff:
  ll_vec_clear(supp,n);
  ll_vec_clear(c,n);
  ll_vec_clear(e,n);
  ll_vec_clear(y,n);
  ll_vec_clear(f,k);
  ll_mat_free(q,L + 2,m * n + 1);
}

int main(void) {
  srand(time(0));
  mtest_gs_GF17(100);
  mtest_gs_GF64(100);
  return 0;
}

